/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.collections;

import java.util.ArrayList;
import java.util.List;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
/**
 * This class contains static list helper methods that aren't implemented in apache commmons (with
 * generics).
 *
 * @author Ronnie Northup
 *
 */
public class ListUtils {

	/**
	 * Returns a list of all elements of the source list that are not present in the other list.
	 *
	 * @param <T> Can be any type that implements equals(Object) to uniquely identify the object.
	 * @param source
	 * @param other
	 * @return
	 */
	public static <T> List<T> complement(List<T> source, List<T> other)
	{
		List<T> results = new ArrayList<T>();
		for (T value : source) {
			if (! other.contains(value)) {
				results.add(value);
			}
		}
		return results;
	}

	/**
	 * Returns a list of all elements in source that are also present in other.
	 *
	 * @param <T>
	 * @param source
	 * @param other
	 * @return
	 */
	public static <T> List<T> intersection(List<T> source, List<T> other)
	{
		List<T> results = new ArrayList<T>();
		for (T value : source) {
			if (other.contains(value)) {
				results.add(value);
			}
		}
		return results;
	}

	/**
	 * Returns a list that is the distinct combination of all elements from both lists.
	 * @param <T>
	 * @param source
	 * @param other
	 * @return
	 */
	public static <T> List<T> union(List<T> source, List<T> other)
	{
		List<T> results = new ArrayList<T>(source);
		for (T value : other) {
			if (! source.contains(value)) {
				results.add(value);
			}
		}
		return results;
	}

}
