/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.dao;

import org.apache.commons.lang.ArrayUtils;

import java.io.Serializable;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public class DataNotFoundException extends DataAccessException
{
	private Class<?> entityType;
	private Serializable id;
	private Predicate[] predicates;

	public DataNotFoundException() {
	}

	public DataNotFoundException(Class<?> entityType, Serializable id)
	{
		super(entityType.getName() + ": " + id);
		this.entityType = entityType;
		this.id = id;
	}

	public DataNotFoundException(Class<?> entityType, Predicate... predicates)
	{
		super(entityType.getName() + ": " + ArrayUtils.toString(predicates));
		this.entityType = entityType;
		this.predicates = predicates;
	}

	public Class<?> getEntityType()
	{
		return entityType;
	}

	public void setEntityType(Class<?> entityType)
	{
		this.entityType = entityType;
	}

	public Serializable getId()
	{
		return id;
	}

	public void setId(Serializable id)
	{
		this.id = id;
	}
}