package com.portlandwebworks.commons.dao;

import java.util.ArrayList;
import java.util.List;

/**
 * This software is the property of Portland Webworks.
 *
 * This search criteria implementation serves several purposes:
 *
 * <ul>
 *     <li>
 *         It supports fielding all filters, sorting, and paging specifications that can be issued by an Ext JS 4 data
 *         store.  This implementation however is somewhat limited in functionality however, because the Ext JS filter
 *         specification does not support all operations implemented in the Predicate API.
 *     </li>
 *     <li>
 *         It provides a way to implement a service's search operation without having to create a class that defines the
 *         searchable @SearchCriterion annotated fields.  Under the hood, the EntityManager converts @SearchCriterion
 *         annotated search fields into Predicates, so this subclass of SearchCriteria basically bypasses this conversion.
 *     </li>
 *     <li>
 *         It provides a service to expose a search API that can be easily extended without overriding the search method.
 *         This is because the list of predicates cna contain any arbitrary set of search operations.
 *     </li>
 * </ul>
 *
 * @author Ronnie Northup
 */
public class PredicateSearchCriteria extends PagedSearchOptions implements SearchCriteria
{
	private List<Predicate> filter = new ArrayList<Predicate>();

	public List<Predicate> getFilter()
	{
		return filter;
	}

	public void setFilter(List<Predicate> filter)
	{
		this.filter = filter;
	}

}
