package com.portlandwebworks.commons.data;

import com.portlandwebworks.commons.dao.IEntityManager;
import com.portlandwebworks.commons.domain.Identifiable;

import java.util.List;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public interface IEntityHandler
{
	/**
	 * Dependency injector for performing database operations.
	 *
	 * @param entityManager
	 */
	void setEntityManager(IEntityManager entityManager);

	/**
	 * An new, unpersisted entity has been instantiated but not yet populated.
	 *
	 * @param entity
	 */
	void entityCreated(Identifiable entity);

	/**
	 * A previously persisted entity has been loaded from the database, prior to any field updates being made.
	 *
	 * @param entity
	 */
	void entityLoaded(Identifiable entity);

	/**
	 * A new value has been assigned to an entity.
	 *
	 * @param entity
	 * @param path
	 * @param oldValue
	 * @param newValue
	 */
	void entityFieldValueApplied(Identifiable entity, String path, Object oldValue, Object newValue);

	/**
	 * An entity has been associated with a parent entity.
	 *
	 * @param identifiable
	 * @param path
	 * @param entity
	 */
	void entityAssignedToParent(Identifiable identifiable, String path, Identifiable entity);

	/**
	 * One or more entities have been associated with a parent entity.
	 *
	 * @param entity
	 * @param path
	 * @param entities
	 */
	void entitiesAssignedToParent(Identifiable entity, String path, List<Identifiable> entities);

	/**
	 * An entity has been unassigned from a one-to-one relationship on a parent entity.
	 *
	 * @param peek
	 * @param path
	 * @param entity
	 */
	void entityUnassignedFromParent(Identifiable peek, String path, Identifiable entity);

	/**
	 * One or more entities have been loaded from their data store.
	 *
	 * @param entities
	 */
	void entitiesLoaded(List<Identifiable> entities);

	/**
	 * One or more entities has been removed from a one-to-many relationship on a parent entity.
	 *
	 * @param parent
	 * @param path
	 * @param entities
	 */
	void entitiesUnassignedFromParent(Identifiable parent, String path, List<Identifiable> entities);

	/**
	 * All fields of an entity have been populated from the source bean.  This does not indicate that any of the fields
	 * on the entity have changed.  Use the other callbacks to handle changes to individual fields.
	 *
	 * @param entity
	 */
	void entityUpdated(Identifiable entity);
}
