package com.portlandwebworks.commons.data.handlers;

import com.portlandwebworks.commons.domain.AuditedEntity;
import com.portlandwebworks.commons.domain.Identifiable;
import com.portlandwebworks.commons.domain.User;
import com.portlandwebworks.commons.security.ISecurityProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;

/**
 * This software is the property of Portland Webworks.
 *
 * This entity handler adds support for setting the audit related fields on entities when they are modified.
 *
 * Do not use this handler directly or add it to an IEntityHandler instance.  It is considered a "built-in" handler that
 * is always present, as it provides necessary functionality.
 *
 * @author Ronnie Northup
 */
public class AuditedEntityHandler extends DefaultEntityHandler
{
	private static Logger logger = LoggerFactory.getLogger(AuditedEntityHandler.class);

	@Autowired
	ISecurityProvider securityProvider;

	public void setSecurityProvider(ISecurityProvider securityProvider)
	{
		this.securityProvider = securityProvider;
	}

	@Override
	public void entityCreated(Identifiable entity)
	{
		Date date = new Date();
		if (AuditedEntity.class.isAssignableFrom(entity.getClass())) {
			AuditedEntity audited = (AuditedEntity)entity;
			User user = (User)securityProvider.getPrincipal();
			audited.setCreatedDate(date);
			audited.setLastModifiedDate(date);
			audited.setCreatingUser(user);
			audited.setLastModifyingUser(user);
		}
		else if (User.class.isAssignableFrom(entity.getClass())) {
			User audited = (User)entity;
			User user = (User)securityProvider.getPrincipal();
			audited.setCreatedDate(date);
			audited.setLastModifiedDate(date);
			audited.setCreatingUserId(user.getId());
			audited.setLastModifyingUserId(user.getId());
		}
	}

	@Override
	public void entityUpdated(Identifiable entity)
	{
		if (AuditedEntity.class.isAssignableFrom(entity.getClass())) {
			AuditedEntity audited = (AuditedEntity)entity;
			User user = (User)securityProvider.getPrincipal();
			audited.setLastModifiedDate(new Date());
			audited.setLastModifyingUser(user);
		}
		else if (User.class.isAssignableFrom(entity.getClass())) {
			User audited = (User)entity;
			User user = (User)securityProvider.getPrincipal();
			audited.setLastModifiedDate(new Date());
			audited.setLastModifyingUserId(user.getId());
		}
	}
}
