package com.portlandwebworks.commons.data.handlers;

import com.portlandwebworks.commons.dao.IEntityManager;
import com.portlandwebworks.commons.data.IEntityHandler;
import com.portlandwebworks.commons.domain.Identifiable;

import java.util.List;

/**
 * This software is the property of Portland Webworks.
 *
 * This handler provides an entity builder with the ability to track whether an entity being updated, or any of its
 * child entities, has been modified due to new field values.
 *
 * Do not use this handler directly or add it to an IEntityHandler instance.  It is considered a "built-in" handler that
 * is always present, as it provides necessary functionality.
 *
 * @author Ronnie Northup
 */
public class EntityModificationTracker implements IEntityHandler
{

	private boolean modificationDetected;

	public boolean isModificationDetected()
	{
		return modificationDetected;
	}

	public void setEntityManager(IEntityManager entityManager)
	{
	}

	public void entityCreated(Identifiable entity)
	{
		modificationDetected = true;
	}

	public void entityLoaded(Identifiable entity)
	{
	}

	public void entityFieldValueApplied(Identifiable entity, String path, Object oldValue, Object newValue)
	{
		modificationDetected = true;
	}

	public void entityAssignedToParent(Identifiable identifiable, String path, Identifiable entity)
	{
		modificationDetected = true;
	}

	public void entitiesAssignedToParent(Identifiable entity, String path, List<Identifiable> entities)
	{
		modificationDetected = true;
	}

	public void entityUnassignedFromParent(Identifiable peek, String path, Identifiable entity)
	{
		modificationDetected = true;
	}

	public void entitiesLoaded(List<Identifiable> entities)
	{
	}

	public void entitiesUnassignedFromParent(Identifiable parent, String path, List<Identifiable> entities)
	{
		modificationDetected = true;
	}

	public void entityUpdated(Identifiable entity)
	{
		modificationDetected = true;
	}
}
