/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.domain;

import java.io.Serializable;

/**
 * This software is the property of Portland Webworks.
 *
 * Superclass for all objects that have an identifier making the object unique for all instances of that type.
 *
 *
 * @Author rnorthup
 */
public abstract class Identifiable<T extends Serializable> implements Serializable
{
	/**
	 * 
	 * @return
	 */
	public abstract T getId();

	/**
	 *
	 * @param id
	 */
	public abstract void setId(T id);


	@Override
	public boolean equals(Object o)
	{
		if (this == o) return true;
		if (!(o instanceof Identifiable)) return false;

		Identifiable iEntity = (Identifiable)o;

		if (getId() != null ? !getId().equals(iEntity.getId()) : iEntity.getId() != null) return false;

		return true;
	}

	@Override
	public int hashCode()
	{
		T id = getId();
		return id != null ? id.hashCode() : 0;
	}

	@Override
	public String toString()
	{
		return this.getClass().getSimpleName() + "(id=" + getId() + ")";
	}
}
