package com.portlandwebworks.commons.hibernate;

import com.portlandwebworks.commons.dao.IEntityManager;
import com.portlandwebworks.commons.domain.AuditedEntity;
import com.portlandwebworks.commons.domain.User;
import com.portlandwebworks.commons.security.ISecurityProvider;
import org.apache.commons.lang.ArrayUtils;
import org.hibernate.*;
import org.hibernate.criterion.Restrictions;
import org.hibernate.type.Type;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;

import java.io.Serializable;
import java.util.Date;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public class AuditInterceptor extends EmptyInterceptor implements BeanFactoryAware
{
	private BeanFactory beanFactory;

	public void setBeanFactory(BeanFactory beanFactory) throws BeansException
	{
	   this.beanFactory = beanFactory;
	}
	
	@Override
	public boolean onSave(Object entity, Serializable id, Object[] state, String[] propertyNames, Type[] types) {
		if (!shouldAudit(entity)) {
			return true;
		}
		User currentUser = getSessionUser();
		Date modificationDate = new Date();
		setValue(state, propertyNames, "createdDate", modificationDate);
		setValue(state, propertyNames, "creatingUser", currentUser);
		setValue(state, propertyNames, "lastModifiedDate", modificationDate);
		setValue(state, propertyNames, "lastModifyingUser", currentUser);
		return true;
	}

	private boolean shouldAudit(Object entity) {
		return entity instanceof AuditedEntity;
	}

	@Override
	public boolean onFlushDirty(Object entity, Serializable id, Object[] state, Object[] previousState, String[] propertyNames, Type[] types) {
		if (!shouldAudit(entity)) {
			return true;
		}
		User currentUser = getSessionUser();
		Date modificationDate = new Date();
		setValue(state, propertyNames, "lastModifiedDate", modificationDate);
		setValue(state, propertyNames, "lastModifyingUser", currentUser);
		return true;
	}

	private User getSessionUser() {
		User user = null;
		ISecurityProvider securityProvider = beanFactory.getBean(ISecurityProvider.class);
		IEntityManager entityManager = beanFactory.getBean(IEntityManager.class);
		SessionFactory factory = ((Session)entityManager.getPersistenceContext()).getSessionFactory();
		StatelessSession session = factory.openStatelessSession();
		try {
			session.beginTransaction();
			Criteria criteria = session.createCriteria(User.class);
			criteria.add(Restrictions.eq("login", securityProvider.getPrincipal()));
			user = (User) criteria.list().get(0);
		}
		finally {
			session.getTransaction().commit();
			session.close();
		}
		return user;
	}


	private void setValue(Object[] currentState, String[] propertyNames, String propertyToSet, Object value) {
		int index = ArrayUtils.indexOf(propertyNames, propertyToSet);
		if (index >= 0) {
			currentState[index] = value;
		}
	}


}