package com.portlandwebworks.commons.pdf;

import com.portlandwebworks.commons.template.ITemplate;
import com.portlandwebworks.commons.template.ITemplateProvider;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.w3c.dom.Document;
import org.xhtmlrenderer.layout.SharedContext;
import org.xhtmlrenderer.pdf.ITextRenderer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.Map;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 * @author John Gordon
 */
public class PdfBuilder implements InitializingBean
{
	@Autowired(required = false)
	private ITemplateProvider templateProvider;

	/**
	 * The component that is responsible for loading and supplying this PDF builder with templates for rendering the HTML
	 * necessary to generate PDF.
	 *
	 * @param templateProvider
	 */
	public void setTemplateProvider(ITemplateProvider templateProvider)
	{
		this.templateProvider = templateProvider;
	}

	/**
	 *
	 * @param templateName
	 * @param data
	 * @return
	 */
	public byte[] buildPdf(String templateName, Map<String, Object> data)
	{
		ITemplate template = templateProvider.getTemplate(templateName);

		template.setParamValues(data);
		
		String xhtml = template.getBody();

		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

		try {
			DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			InputStream stream = new ByteArrayInputStream(xhtml.getBytes("UTF-8"));

			Document doc = builder.parse(stream);

			ITextRenderer renderer = new ITextRenderer();

			SharedContext context = renderer.getSharedContext();

			ClasspathReplacedElementFactory customReplacementFactory = new ClasspathReplacedElementFactory(context);

			context.setReplacedElementFactory(customReplacementFactory);

			renderer.setDocument(doc, null);

			renderer.layout();
			
			renderer.createPDF(outputStream);

			return outputStream.toByteArray();
		} 
		catch (Exception e) {
			throw new RuntimeException(e.getMessage(), e);
		}

	}

	public void afterPropertiesSet() throws Exception
	{
		if (templateProvider == null) {
			throw new RuntimeException("template provider is required");
		}
	}
}
