package com.portlandwebworks.commons.reflection;


/**
 * This software is the property of Portland Webworks.
 *
 * A "Node" is considered any location in an object graph.  Simply put, it is a field on a class.  The purpose of NodeType
 * is to give some indication as to whether the field can be traversed to another node in the object graph or whether is
 * is a "leaf".  ValueType is a leaf.  Object might be a leaf if it only contains ValueType fields.  Array, Collection and
 * Map are leaves if they contain no members.
 *
 * @author Ronnie Northup
 */
public enum NodeType
{
	/**
	 * This value represents a single value:
	 * <ul>
	 *      <li>A Java primitive type</li>
	 *      <li>An object wrapper for a primitive type</li>
	 *      <li>A java.util.String</li>
	 *      <li>A java.util.Date or java.sql.Timestamp</li>
	 *      <li>A Java enumeration</li>
	 * </ul>
	 */
	ValueType,

	/**
	 * Any Class that does not meet any of the ValueType criteria
	 */
	Object,

	/**
	 * A Java array of any node type
	 */
	Array,

	/**
	 * A collection of any type
	 */
	Collection,

	/**
	 * A map of any key or value type
	 */
	Map
}
