package com.portlandwebworks.commons.security.user;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.enums.EnumUtils;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;


/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public class DefaultPasswordGenerator implements IPasswordGenerator
{
	public String generateTemporaryPassword()
	{
		return RandomStringUtils.randomAlphanumeric(8);
	}

	public PasswordStrength getPasswordStrength(String password)
	{
		if (password.length() < 6) {
			return PasswordStrength.Unacceptable;
		}

		int hasCaps = password.toLowerCase().equals(password)? 0: 1;
		int hasNumeric = StringUtils.containsAny(password, "1234567890")? 1: 0;
		int hasSpecial = StringUtils.containsAny(password, "!@#$%^&*")? 1: 0;
		int hasLength = password.length() >= 8? 1: 0;
		
		int numericStrength = hasCaps + hasNumeric + hasSpecial + hasLength;
		
		switch (numericStrength) {
			case 0: case 1: return PasswordStrength.Weak;
			case 2: return PasswordStrength.Moderate;
			default: return PasswordStrength.Strong;
		}
	}

	public String getPasswordHash(String password)
	{
		MessageDigest md = null;
		try {
			md = MessageDigest.getInstance("MD5");
		}
		catch (NoSuchAlgorithmException ignore) {
		}

		assert md != null;

		md.update(password.getBytes(), 0, password.length());

		return String.valueOf(Hex.encodeHex(md.digest()));
	}
}
