/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.security.user;

import com.portlandwebworks.commons.dao.PredicateSearchCriteria;
import com.portlandwebworks.commons.domain.Role;
import com.portlandwebworks.commons.domain.User;

import java.util.List;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
public interface IUserDelegate
{
	Class<? extends Role> getRoleType();

	Class<? extends User> getUserType();

	Class<? extends Enum> getFetchOptionsType();
	
	Class<? extends UserInput> getUserInputType();

	Class<? extends UserInputOptions> getUserInputOptionsType();
	
	Class<? extends UserSearchOptions> getUserSearchOptionsType();

	Class<? extends UserSearchResult> getUserSearchResultType();

	/**
	 * P
	 * @param input
	 */
	void prepareUserInputForEdit(UserInput input);

	/**
	 * Provides an opportunity to set any custom fields on a new user prior to adding them to the system.
	 * 
	 * @param input
	 * @param user
	 */
	void prepareUserForAdd(UserInput input, User user);

	/**
	 * Provides an opportunity to set any custom fields on a user prior to committing changes to the database.
	 *
	 * @param input
	 * @param user
	 */
	void prepareUserForUpdate(UserInput input, User user);

	/**
	 * Use this method to populate backing data for any additional multi-value fields on a custom user input form.
	 *
	 * @param userId
	 * @param options
	 */
	void prepareUserInputOptions(Long userId, UserInputOptions options);

	/**
	 * Use this method to populate backing data for any additional multi-value fields on a custom user search form
	 * @param uso
	 */
	void prepareUserSearchOptions(UserSearchOptions uso);

	/**
	 * Callback executed after a user is successfully added to the system.
	 * 
	 * @param input
	 * @param user
	 */
	void userAdded(UserInput input, User user);

	/**
	 * Callback executed after a user is updated in the system.
	 * @param user
	 * @param input
	 */
	void userUpdated(User user, UserInput input);

	/**
	 *
	 * @param user
	 */
	void prepareUserForView(User user);

	/**
	 * Allows a delegate to tailor search results for a user subclass.  This method is called within a database
	 * transaction.
	 *
	 * @param items
	 */
	void prepareUserSearchResults(List<? extends UserSearchResult> items);

	/**
	 * Retrieves a set of lookup values for a specific field on a user input form, for a specific user, or for any user
	 * if null is specified for user ID.
	 *
	 *
	 * @param optionName
	 * @param qualifier
	 * @return
	 */
	List getUserInputOptions(String optionName, String qualifier);

	/**
	 * Allows the delegate to make any desired modifications, validation, or additions to the search criteria before
	 * applying them to the underlying search.
	 *
	 * @param criteria
	 */
	void prepareUserSearchCriteria(PredicateSearchCriteria criteria);

	List getUserSearchOptions(String optionName, String qualifier);
}
