/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.security.user;

import com.portlandwebworks.commons.data.EntityMapping;
import com.portlandwebworks.commons.data.FieldMapping;
import com.portlandwebworks.commons.reflection.FieldUtils;
import com.portlandwebworks.commons.validation.Matches;
import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotNull;
import java.lang.reflect.Field;
import java.util.*;

/**
 * This software is the property of Portland Webworks.
 *
 * @author Ronnie Northup
 */
@EntityMapping("com.portlandwebworks.commons.domain.User")
@Matches(field = "password", verifyField = "confirmPassword")
public class UserInput implements Map<String, Object>
{
	@FieldMapping(value = "id", identifier = true)
	private Long userId;

	@Length(max = 30)
	@NotNull
	@FieldMapping(groups = "admin")
	private String username;

	@Length(max = 30)
	@NotNull
	@FieldMapping
	private String firstName;

	@Length(max = 30)
	@NotNull
	@FieldMapping
	private String lastName;

	@FieldMapping
	private Character middleInitial;

	@Length(max = 10)
	@FieldMapping
	private String suffix;

	@NotNull
	@FieldMapping(value = "roles.id", identifier = true, groups = "admin")
	private List<Integer> roleIds;

	@FieldMapping(groups = "admin")
	private boolean enabled;

	@FieldMapping(ignoreNull = true)
	@Length(max = 15)
	private String password;

	@Length(max = 15)
	private String confirmPassword;

	@Length(max = 120)
	@Email
	@NotNull
	@FieldMapping
	private String email;
	
	@FieldMapping(readOnly = true)
	private Date createdDate;
	
	@FieldMapping(readOnly = true)
	private Date lastLoginDate;
	
	@FieldMapping(readOnly = true)
	private Date lastModifiedDate;

	@FieldMapping(readOnly = true)
	private Long lastModifyingUserId;
	
	private Map<String, Object> customFields = new HashMap<String, Object>();


	public Long getUserId()
	{
		return userId;
	}

	public void setUserId(Long userId)
	{
		this.userId = userId;
	}

	public String getUsername()
	{
		return username;
	}

	public void setUsername(String username)
	{
		this.username = username;
	}

	public String getFirstName()
	{
		return firstName;
	}

	public void setFirstName(String firstName)
	{
		this.firstName = firstName;
	}

	public String getLastName()
	{
		return lastName;
	}

	public void setLastName(String lastName)
	{
		this.lastName = lastName;
	}

	public boolean isEnabled()
	{
		return enabled;
	}

	public void setEnabled(boolean enabled)
	{
		this.enabled = enabled;
	}

	public String getEmail()
	{
		return email;
	}

	public void setEmail(String email)
	{
		this.email = email;
	}

	public List<Integer> getRoleIds()
	{
		return roleIds;
	}

	public void setRoleIds(List<Integer> roleIds)
	{
		this.roleIds = roleIds;
	}

	public Character getMiddleInitial()
	{
		return middleInitial;
	}

	public void setMiddleInitial(Character middleInitial)
	{
		this.middleInitial = middleInitial;
	}

	public String getSuffix()
	{
		return suffix;
	}

	public void setSuffix(String suffix)
	{
		this.suffix = suffix;
	}

	public String getPassword()
	{
		return password;
	}

	public void setPassword(String password)
	{
		this.password = password;
	}

	public String getConfirmPassword()
	{
		return confirmPassword;
	}

	public void setConfirmPassword(String confirmPassword)
	{
		this.confirmPassword = confirmPassword;
	}

	public Date getCreatedDate()
	{
		return createdDate;
	}

	public void setCreatedDate(Date createdDate)
	{
		this.createdDate = createdDate;
	}

	public Date getLastLoginDate()
	{
		return lastLoginDate;
	}

	public void setLastLoginDate(Date lastLoginDate)
	{
		this.lastLoginDate = lastLoginDate;
	}

	public Date getLastModifiedDate()
	{
		return lastModifiedDate;
	}

	public void setLastModifiedDate(Date lastModifiedDate)
	{
		this.lastModifiedDate = lastModifiedDate;
	}

	public Long getLastModifyingUserId()
	{
		return lastModifyingUserId;
	}

	public void setLastModifyingUserId(Long lastModifyingUserId)
	{
		this.lastModifyingUserId = lastModifyingUserId;
	}



	public int size()
	{
		// Subtract one for "customFields" map field.
		return (FieldUtils.getFields(this.getClass()).length - 1) + (customFields.size());
	}

	public boolean isEmpty()
	{
		return false;
	}

	public boolean containsKey(Object key)
	{
		if (customFields.containsKey(key.toString())) {
			return true;
		}
		
		if (FieldUtils.containsField(this.getClass(), key.toString())) {
			FieldUtils.getField(this.getClass(), key.toString());
		}

		return false;
	}

	public boolean containsValue(Object value)
	{
		if (customFields.containsValue(value)) {
			return true;
		}

		return FieldUtils.getFieldValue(this, value.toString()) != null;
	}

	public Object get(Object key)
	{
		if (customFields.containsKey(key.toString())) {
			return customFields.get(key);
		}

		return FieldUtils.getFieldValue(this, key.toString());
	}

	public Object put(String key, Object value)
	{
		if (FieldUtils.containsField(this.getClass(), key)) {
			Object temp = FieldUtils.getFieldValue(this, key);
			FieldUtils.setConvertedFieldValue(this, key, value);
			return temp;
		}
		else {
			return customFields.put(key, value);
		}
	}

	public Object remove(Object key)
	{
		if (FieldUtils.containsField(this.getClass(), key.toString())) {
			Object temp = FieldUtils.getFieldValue(this, key.toString());
			FieldUtils.setFieldValue(this, key.toString(), null);
			return temp;
		}
		else {
			return customFields.remove(key);
		}
	}

	public void putAll(Map<? extends String, ? extends Object> m)
	{
		for (String key : m.keySet()) {
			if (FieldUtils.containsField(this.getClass(), key)) {
				FieldUtils.setConvertedFieldValue(this, key, m.get(key));
			}
			else {
				customFields.put(key, m.get(key));
			}
		}
	}

	public void clear()
	{
		for (Field field : FieldUtils.getFields(this.getClass())) {
			if (field.getName().equals("customFields")) {
				continue;
			}
			FieldUtils.setFieldValue(this, field, null);
		}
		
		customFields.clear();
	}

	public Set<String> keySet()
	{
		Set<String> set = new HashSet<String>();
		for (Field field : FieldUtils.getFields(this.getClass())) {
			if (field.getName().equals("customFields")) {
				continue;
			}
			set.add(field.getName());
		}

		set.addAll(customFields.keySet());

		return set;
	}

	public Collection<Object> values()
	{
		List<Object> values = new ArrayList<Object>();
		values.addAll(customFields.values());
		for (Field field : FieldUtils.getFields(this.getClass())) {
			if (field.getName().equals("customFields")) {
				continue;
			}
			Object value = FieldUtils.getFieldValue(this, field);
			if (value != null) {
				values.add(value);
			}
		}
		
		values.addAll(customFields.values());
		
		return values;
	}

	public Set<Entry<String, Object>> entrySet()
	{
		Set<Entry<String, Object>> set = new HashSet<Entry<String, Object>>();
		for (Field field : FieldUtils.getFields(this.getClass())) {
			if (field.getName().equals("customFields")) {
				continue;
			}

			set.add(new AbstractMap.SimpleEntry<String, Object>(
					field.getName(), 
					FieldUtils.getFieldValue(this, field))
			);
		}
		
		set.addAll(customFields.entrySet());

		return set;
	}

	public boolean equals(Object o)
	{
		return false;  //To change body of implemented methods use File | Settings | File Templates.
	}

	public int hashCode()
	{
		return 0;  //To change body of implemented methods use File | Settings | File Templates.
	}
}
