/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.servlet;

import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.beanutils.PropertyUtils;

import javax.servlet.Filter;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.Enumeration;

/**
 * This software is the property of Portland Webworks.
 *
 * Applies javabean conventions to filters so that your filter's init parameters are automatically applied to filter
 * fields.
 *
 * @Author rnorthup
 */
public abstract class FilterBean implements Filter
{
	public void init(FilterConfig filterConfig) throws ServletException
	{
		Enumeration<String> paramEnum = filterConfig.getInitParameterNames();
		while (paramEnum.hasMoreElements()) {
			String paramName = paramEnum.nextElement();
			String paramValue = filterConfig.getInitParameter(paramName);

			try {
				Field field = this.getClass().getDeclaredField(paramName);
				
				field.setAccessible(true);

				Object paramObj = ConvertUtils.convert(paramValue, field.getType());

				field.set(this, paramObj);
			}
			catch (IllegalAccessException e) {
				throw new ServletException("Cannot apply init param " + paramName + " to " + this.getClass().getName() + ", inaccessible property");
			}
			catch (NoSuchFieldException e) {
				throw new ServletException("Cannot apply init param " + paramName + " to " + this.getClass().getName() + ", no field available");
			}

		}
	}
}
