/*
 * Copyright 2010 Portland Webworks, Inc.
 *
 * This file is part of PWW Commons.
 *
 * PWW Commons is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PWW Commons is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with PWW Commons.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

package com.portlandwebworks.commons.template;

import com.portlandwebworks.commons.html.HtmlUtils;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.tools.generic.DateTool;

import java.io.IOException;
import java.io.StringWriter;
import java.util.Map;

/**
 * This software is the property of Portland Webworks.
 *
 * @Author rnorthup
 */
public class VelocityTemplate implements ITemplate
{
	private Template vm;

	private VelocityContext context;

	private String renderedTitle;

	private String renderedBody;


	public VelocityTemplate(Template vm)
	{
		this.vm = vm;

		this.context = new VelocityContext();
		this.context.put("dateTool", new DateTool());
	}

	/**
	 * Sets the values that will be used to populate the velocity template.  These key/value pairs are accumulative, so
	 * you can call this method repeatedly with different args.
	 *
	 * @param args
	 */
	public void setParamValues(Map<String, Object> args)
	{
		for (String key : args.keySet()) {
			context.put(key, args.get(key));
		}

		_renderSubjectAndBody();
	}

	private void _renderSubjectAndBody()
	{
		StringWriter writer = new StringWriter();

		try {
			vm.merge(context, writer);

			String emailContent = writer.getBuffer().toString();

			if (HtmlUtils.isContentHtml(emailContent)) {
				// If the email content is HTML, use the contents of the title tag as the subject of the email.
				renderedTitle = HtmlUtils.extractTitleFromHtml(emailContent);
				renderedBody = emailContent;
			}
			else {
				// If this is a plain text email, assume the first line of content is the subject.  Split the content
				// into two parts to create a subject line and a body.
				int endOfLine = emailContent.indexOf("\n");
				if (endOfLine == -1) {
					// This short email has only one line of text in it.  Don't do a subject line.
					renderedTitle = "";
					renderedBody = emailContent;
				}
				else {
					// Use the first line of content as the subject, and the rest as the body.
					renderedTitle = emailContent.substring(0, endOfLine);
					renderedBody = emailContent.substring(endOfLine + 1);
				}
			}
		}
		catch (IOException e) {
			throw new RuntimeException("Unhandled I/O exception rendering email content", e);
		}
	}

	public String getTitle()
	{
		return renderedTitle;
	}

	public String getBody()
	{
		return renderedBody;
	}

	public boolean isHtml()
	{
		return HtmlUtils.isContentHtml(renderedBody);
	}
}
